import asyncio
import json
import os
import platform as pf
import sqlite3
import sys
from pathlib import Path
from typing import Optional


import click
import dotenv
import sqlalchemy
import typer
from click import MissingParameter, UsageError
from dotenv import load_dotenv
from loguru import logger
from rich import print as rprint
from rich.progress import (
	Progress,
	SpinnerColumn,
)
from sqlmodel import SQLModel
from trogon import tui
from typing_extensions import Annotated
import datahorse
import pandas as pd


from . import LOGURU_LEVEL
from .engine_and_database import (
	add_products_to_db,
	add_suppliers_to_db,
	create_db_engine,
	save_all_changes,
)
from .info_message import update_db_success_sqlite, update_db_with_success
from .scrape_from_disk import PageParser
from aba_cli_scrapper.proxies_providers import BrightDataProxyProvider, SyphoonProxyProvider
from .ascii import display_banner, get_current_version

load_dotenv()
logger.remove(0)
logger.add(sys.stderr, colorize=True, level=f"{LOGURU_LEVEL}")  # type: ignore
# display_banner()  # display banner for all commands


@tui(name="text-mode", command="aba-run")
@click.group()
def app():
	pass


app_t = typer.Typer(
	pretty_exceptions_enable=False,
)


@app_t.command()
def version():
	"""
	Display the version of the program
	"""
	rprint(f"[magenta]aba-cli-scrapper version:[/magenta] [blue]{get_current_version()}")


# @app_t.callback(invoke_without_command=True,name="--version", help="Display the version of the program",)
# def version():
# 	rprint(f"[magenta]aba-cli-scrapper version:[/magenta] [blue]{get_current_version()}")


def _db_url(credentials: dict = dict(), auto_fill: bool = False):
	"""
	Return a SQLAlchemy database URL based on the given credentials.

	If `auto_fill` is True, use the credentials from `db_credentials.json` if it exists
	and fill in any missing values from the provided `credentials`. If `db_credentials.json`
	does not exist, raise a UsageError.

	If `auto_fill` is False, use the provided `credentials` to create a new database URL
	and write it to `db_credentials.json`.
	"""
	if auto_fill is True:
		if Path("db_credentials.json").exists() is False:
			raise UsageError(
				"db_credentials.json is missing if you want to auto fill your mysql credentials you need to run `db-init` subcommand at least once with `--user`, `--password` and `--db-name` arguments."
			)
		with open("db_credentials.json", "r") as f:
			cred: dict = json.load(f)
			items = credentials.items()
			for item in items:
				if item[0] == "host" and item[1] not in ["localhost", None]:
					cred.update({"host": item[1]})
				elif item[0] == "port" and item[1] not in [3306, None]:
					cred.update({"port": item[1]})
				elif item[0] == "db_name" and item[1] not in [
					cred.get("db_name"),
					None,
				]:
					cred.update({"db_name": item[1]})
				elif item[0] == "user" and item[1] not in [
					cred.get("user"),
					None,
				]:
					cred.update({"user": item[1]})
				elif item[0] == "password" and item[1] not in [
					cred.get("password"),
					None,
				]:
					cred.update({"password": item[1]})
				else:
					continue
		return f"mysql+pymysql://{cred.get('user')}:{cred.get('password')}@{cred.get('host')}/{cred.get('db_name')}"

	else:
		cred = {
			"user": credentials.get("user"),
			"password": credentials.get("password"),
			"host": credentials.get("host"),
			"port": credentials.get("port"),
			"db_name": credentials.get("db_name"),
		}
		for key, value in cred.items():
			if value is None:
				raise MissingParameter(f"{key} is missing")
		with open("db_credentials.json", "w") as f:
			json.dump(cred, f)
		return f"mysql+pymysql://{cred.get('user')}:{cred.get('password')}@{cred.get('host')}/{cred.get('db_name')}"


@app_t.command()
def scraper(
	key_words: Annotated[str, typer.Argument(help="Keywords to search for on alibaba")] = "",
	html_folder: Annotated[
		Optional[str],
		typer.Option("--html-folder", "-hf", help="Folder to save the results"),
	] = None,
	sync_api: Annotated[
		Optional[bool],
		typer.Option("--sync-api", "-sa", help="wether to sync or not"),
	] = False,
	page_results: Annotated[
		int,
		typer.Option(
			"--page-results",
			"-pr",
			help="Number of results per page to scrape from alibaba 10 by defaults",
		),
	] = 10,
) -> None:
	"""
	Scrape with Bright-Data proxies rovider.
	"""
	display_banner()  # display banner for all commands
	save_in_folder = (
		key_words.strip().replace(" ", "_")
		if (html_folder is None and html_folder != "")
		else html_folder
	)
	if sync_api:
		BrightDataProxyProvider.sync_scraper(
			save_in=save_in_folder,
			key_words=key_words,
			page_results=page_results,
		)
	else:
		asyncio.run(
			BrightDataProxyProvider.async_scraper(
				save_in=save_in_folder, key_words=key_words, page_results=page_results
			)
		)


@app_t.command()
def syphoon_scraper(
	key_words: Annotated[str, typer.Argument(help="Keywords to search for on alibaba")] = "",
	html_folder: Annotated[
		Optional[str],
		typer.Option("--html-folder", "-hf", help="Folder to save the results"),
	] = None,
	sync_api: Annotated[
		Optional[bool],
		typer.Option("--sync-api", "-sa", help="wether to sync or not"),
	] = False,
	page_results: Annotated[
		int,
		typer.Option(
			"--page-results",
			"-pr",
			help="Number of results per page to scrape from alibaba 10 by defaults",
		),
	] = 10,
):
	"""
	Scrape with Syphoon proxies provider.
	"""
	display_banner()  # display banner for all commands
	save_in_folder = (
		key_words.strip().replace(" ", "_")
		if (html_folder is None and html_folder != "")
		else html_folder
	)
	if sync_api:
		SyphoonProxyProvider.sync_scraper(
			save_in=save_in_folder,
			key_words=key_words,
			page_results=page_results,
		)
	else:
		asyncio.run(
			SyphoonProxyProvider.async_scraper(
				save_in=save_in_folder, key_words=key_words, page_results=page_results
			)
		)


@app_t.command()
def db_update(
	kw_results: Annotated[
		Path,
		typer.Option(
			"--kw-results",
			"-kr",
			help="Folder where the html results are stored",
		),
	],
	db_engine: Annotated[
		Optional[str], typer.Argument(help="Name of database engine to use")
	] = "sqlite",
	filename: Annotated[
		Optional[str],
		typer.Option(
			"--filename",
			"-f",
			help="Name of the sqlite file(without any extensions) to update with news data",
		),
	] = None,
	host: Annotated[
		Optional[str],
		typer.Option(
			"--host",
			"-h",
			help="Host of the database engine",
		),
	] = "localhost",
	port: Annotated[
		Optional[int],
		typer.Option("--port", "-p", help="Port of the database engine"),
	] = 3306,
	user: Annotated[
		Optional[str],
		typer.Option(
			"--user",
			"-u",
			help="User of the database engine",
			show_default=False,
		),
	] = None,
	password: Annotated[
		Optional[str],
		typer.Option(
			"--password",
			"-pw",
			help="Password of the database engine",
			show_default=False,
		),
	] = None,
	db_name: Annotated[
		Optional[str],
		typer.Option(
			"--db-name",
			"-db",
			help="Database of the database engine",
			show_default=False,
		),
	] = None,
):
	"""Update a database with scraped products and suppliers data."""
	display_banner()  # display banner for all commands
	if db_engine not in ["sqlite", "mysql"]:
		raise MissingParameter("--engine should be sqlite or mysql")
	if db_engine == "sqlite" and filename is None:
		raise MissingParameter("--filename is required for sqlite engine")
	if db_engine != "sqlite" and filename is not None:
		raise MissingParameter("You don't need to specify --filename for non-sqlite engine")
	if db_engine == "mysql":
		db_url = _db_url(credentials=locals(), auto_fill=True)
		mysql_engine = create_db_engine(db_url=db_url)
		page_parser = PageParser(targeted_folder=kw_results.resolve())
		suppliers = page_parser.detected_suppliers()
		products = page_parser.detected_products()
		add_suppliers_to_db(suppliers=suppliers, engine_db=mysql_engine)
		add_products_to_db(products=products, engine_db=mysql_engine)
		save_all_changes(engine_db=mysql_engine, sql_model=SQLModel)
		update_db_with_success()
		return None
	if not kw_results.exists():
		raise UsageError(f"Folder {kw_results} does not exist")
	else:
		page_parser = PageParser(targeted_folder=kw_results)
		suppliers = page_parser.detected_suppliers()
		products = page_parser.detected_products()
		sqlite_engine = create_db_engine(db_name=filename)  # type: ignore
		add_suppliers_to_db(suppliers=suppliers, engine_db=sqlite_engine)
		add_products_to_db(products=products, engine_db=sqlite_engine)
		save_all_changes(engine_db=sqlite_engine, sql_model=SQLModel)
		update_db_success_sqlite(sqlite_file=filename)  # type: ignore


@app_t.command()
def db_init(
	engine: Annotated[str, typer.Argument(help="Name of database engine to use")] = "sqlite",
	sqlite_file: Annotated[
		Optional[str],
		typer.Option(
			"--sqlite-file",
			"-f",
			help="Name of the sqlite file to use (without any extensions)",
			show_default=False,
		),
	] = None,
	host: Annotated[
		Optional[str],
		typer.Option(
			"--host",
			"-h",
			help="Host of the database engine",
		),
	] = "localhost",
	port: Annotated[
		Optional[int],
		typer.Option("--port", "-p", help="Port of the database engine"),
	] = 3306,
	user: Annotated[
		Optional[str],
		typer.Option("--user", "-u", help="User of the database engine"),
	] = None,
	password: Annotated[
		Optional[str],
		typer.Option("--password", "-pw", help="Password of the database engine"),
	] = None,
	db_name: Annotated[
		Optional[str],
		typer.Option(
			"--db-name",
			"-db",
			help="Database of the database engine",
		),
	] = None,
	only_with: Annotated[
		Optional[bool],
		typer.Option(
			"--only-with",
			"-ow",
			help="set it to true if you just want to update some database crendentials but not all",
		),
	] = False,
):
	"""
	Create a new database sqlite/mysql with `<products>` and `<suppliers>` as tables in it.
	"""
	display_banner()  # display banner for all commands
	if engine not in ["sqlite", "mysql"]:
		raise MissingParameter("--engine should be sqlite or mysql")
	if engine == "sqlite" and sqlite_file is None:
		raise MissingParameter("--sqlite-file is required for sqlite engine")
	if engine != "sqlite" and sqlite_file is not None:
		raise MissingParameter("You dont need to specify --sqlite-file for non-sqlite engine")
	if engine == "mysql":
		if only_with:
			db_url = _db_url(credentials=locals(), auto_fill=True)
		else:
			db_url = _db_url(credentials=locals())
		mysql_engine = create_db_engine(db_url=db_url)
		try:
			save_all_changes(engine_db=mysql_engine, sql_model=SQLModel)
			rprint(
				f"[bold white]Database [magenta bold] {db_name} [/magenta bold] has been created succesfully :white_heavy_check_mark-emoji: ![/bold white]"
			)
		except sqlalchemy.exc.OperationalError as e:
			if "Unknown database" in str(e):
				rprint(
					f"[bold white blink] It's seems like you want to use mysql engine but you haven't created database: [magenta bold] `{db_name}` [/magenta bold] ![/bold white blink] create it first and then try again"
				)
			logger.error(f"Errors has occured: {e}")
	else:
		sqlite_engine = create_db_engine(db_name=sqlite_file)  # type: ignore
		save_all_changes(engine_db=sqlite_engine, sql_model=SQLModel)
		rprint(
			f"[bold white]Database [magenta bold] {sqlite_file}.sqlite [/magenta bold] has been created succesfully :white_heavy_check_mark-emoji: ![/bold white]"
		)


@app_t.command()
def export_as_csv(
	sqlite_file: Annotated[
		str,
		typer.Argument(
			help="take name of the sqlite file",
		),
	],
	to: Annotated[
		str,
		typer.Option(
			"--to",
			"-t",
			help="take name of the csv file",
		),
	],
):
	"""Exports a sqlite database as a csv file. A `FULL OUTER JOIN` operation will be used to join the two tables."""
	display_banner()  # display banner for all commands
	query = """
      SELECT Product.id as product_id,
      Product.name as product_name,
      Product.alibaba_guranteed as alibaba_guranteed,
      Product.minimum_to_order as minimum_to_order,
      Product.supplier_id as related_supplier_id,
      Product.certifications as certifications,
      Product.ordered_or_sold as ordered_or_sold,
      Product.product_score as product_score,
      Product.review_count  as review_count,
      Product.review_score as review_score,
      Product.shipping_time_score as shipping_time_score,
      Product.is_full_promotion as is_full_promotion,
      Product.is_customizable as is_customizable,
      Product.is_instant_order as is_instant_order,
      Product.trade_product  as trade_product,
      Product.min_price as min_price,
      Product.max_price as max_price,
      supplier.name as supplier_name ,
      supplier.verification_mode,
      supplier.sopi_level as sopi_level,
      supplier.country_name as country_name,
      supplier.years_as_gold_supplier as years_as_gold_supplier,
      supplier.supplier_service_score as supplier_service_score,
      supplier.id as supplier_id
      FROM Product
      JOIN Supplier ON Product.supplier_id = Supplier.id"""
	try:
		connector = sqlite3.connect(f"{sqlite_file}")
		df = pd.read_sql_query(query, connector)
		df.to_csv(
			f"{to}",
			index=True,
			encoding="utf-8",
			header=True,
			index_label="supplier_id",
		)
	except pd.errors.DatabaseError as e:
		raise UsageError(
			f"{str(e).split(':')[1] + str(e).split(':')[2]}. Your database must be initialized first run : aba db-init <{sqlite_file}>  "
		)
		return typer.Exit(1)
	rprint(
		f"[bold white] {to} file has been created with success :white_heavy_check_mark-emoji: ![/bold white]"
	)


@app_t.command()
def set_api_key(
	proxies_provider: Annotated[
		str,
		typer.Argument(
			help="take proxies provider you want to use (i.e: Syphoon, Brightdata, etc.)",
		),
	],
	api_key: Annotated[
		str,
		typer.Option(
			help="take api key from selected proxies provider (i.e: Brightdata, Syphoon)",
			prompt=True,
			hide_input=True,
		),
	],
) -> None:
	"""Sets your proxies provider api key could beBrightdata or Syphoon."""
	display_banner()  # display banner for all commands
	dotenv_file = dotenv.find_dotenv()
	dotenv.load_dotenv(dotenv_file)
	value = ""
	if proxies_provider == "brightdata":
		os.environ["BRIGHT_DATA_API_KEY"] = api_key
		value = "BRIGHT_DATA_API_KEY"
	elif proxies_provider == "syphoon":
		os.environ["SYPHOON_API_KEY"] = api_key
		value = "SYPHOON_API_KEY"
	else:
		return
	dotenv.set_key(dotenv_file, f"{value}", os.environ[f"{value}"])
	rprint(
		"[bold white]API key has been saved with success now you can use [magenta bold] `scraper` [/magenta bold] subcommand with async mode  :white_heavy_check_mark-emoji: ![/bold white]"
	)
	rprint(
		"[bold white]You can now use [magenta bold] `scraper` [/magenta bold]  subcommand with async mode with success :white_heavy_check_mark-emoji: ![/bold white]"
	)


@app_t.command()
def ai_agent(
	query: Annotated[
		str,
		typer.Argument(
			help="query that you want to ask to the ai agent",
		),
	],
	csv_file: Annotated[
		str,
		typer.Option(
			"--csv-file",
			"-f",
			help="take name of the csv file that you want to use to chat with the ai agent",
		),
	],
) -> None:
	"""Interact with a csv file provided by you to answer your queries."""
	df = datahorse.read(f"{csv_file}")
	if df is None:
		raise UsageError("An unexpected error has occured. May due to your csv file.")

	display_banner()  # display banner for all commands
	with Progress(
		SpinnerColumn(
			finished_text="[bold green]finished ✓[/bold green]",
		),
		# *Progress.get_default_columns(),
		transient=False,
	) as progress:
		task = progress.add_task(
			description="[green]Processing [blue][/blue] ...",
			start=False,
			total=100,
		)
		df_result = None
		try:
			progress.start_task(task)
			df_result = df.chat(  # type: ignore
				f"{query}"
				+ "return the result as a dataframe with only columns that are explicitly or implicitly asked."
			)  # type: ignore

		except Exception as e:
			raise UsageError(
				f"ai-agent : < An unexpected error has occured: {e}. \n Maybe i miss understood your query.change it a bit, or try the same again.>"
			)
		if type(df_result) is None:
			raise UsageError("An unexpected error has occured. May due to your query.")

		with open("buffer.csv", "w", encoding="utf-8") as output_file:
			df_result.to_csv(output_file)
		progress.advance(task, 100)
		progress.stop()
		os.system("rich buffer.csv --center")
		try:
			if pf.system() != "Windows":
				os.system("rm buffer.csv")
			else:
				os.system("del buffer.csv")
		except Exception as e:
			raise UsageError(
				f"ai-agent : << An unexpected error has occured: {e}. Remove the <buffer.csv> file manually. >>"
			)


typer_click_object = typer.main.get_command(app_t)

app.add_command(typer_click_object, "text-mode")

if __name__ == "__main__":
	app()
	app_t()



from typing import Sequence

from click import UsageError
from loguru import logger
from pymysql import OperationalError as MySQLdbOperationalError
from rich.progress import (
	BarColumn,
	Progress,
	SpinnerColumn,
	TaskProgressColumn,
	TextColumn,
	TimeElapsedColumn,
)
from sqlalchemy.engine import Engine
from sqlalchemy.exc import IntegrityError, OperationalError
from sqlmodel import Session, SQLModel, create_engine, select  # noqa: F401

from aba_cli_scrapper.typed_datas import ProductDict, SupplierDict

from .models import Product, Supplier  # noqa: F401


def create_db_engine(db_name: str = "", db_url: str = ""):
	"""This function creates a new database engine using SQLAlchemy's create_engine method.

	The function takes two arguments: db_name and db_url. db_name is the name of the database
	and db_url is the connection string for the database. If db_url is not provided, the
	function will use a default connection string based on db_name.

	The function returns a new database engine object.

	:param db_name: The name of the database.
	:type db_name: str
	:param db_url: The connection string for the database.
	:type db_url: str
	:return: A new database engine object.
	:rtype: sqlalchemy.engine.Engine
	"""
	db_url = db_url if db_url else f"sqlite:///{db_name}.sqlite"
	return create_engine(db_url)


def save_all_changes(
	engine_db: Engine,
	sql_model,
):
	"""Save all changes to the database.

	This function takes two arguments: engine_db and sql_model. engine_db is the database
	engine object created by create_engine method of SQLAlchemy. sql_model is the
	SQLModel object which is used to access the metadata of the database.

	The function will try to create all tables in the database by calling
	create_all() method of the metadata object. If any error occur during the
	process, it will raise a UsageError with the error message.

	:param engine_db: The database engine object.
	:type engine_db: sqlalchemy.engine.Engine
	:param sql_model: The SQLModel object.
	:type sql_model: sqlmodel.main.SQLModel
	:raises UsageError: If any error occur during the process of creating tables.
	"""
	logger.info("Saving all changes ...")
	try:
		sql_model.metadata.create_all(engine_db)
	except MySQLdbOperationalError as e:
		logger.error(f"Errors has occured: {e}")
		raise UsageError(f"Something went wrong an unexpected error has occured:{e}") from e


def add_suppliers_to_db(suppliers: Sequence[SupplierDict], engine_db: Engine):
	"""
	Add suppliers to the database.

	This function takes two arguments: suppliers and engine_db. suppliers is a list of dictionaries
	containing information about the suppliers. engine_db is the database engine object created by
	create_engine method of SQLAlchemy.

	The function will iterate over the list of suppliers and add each one to the database using the
	Supplier model. If any error occur during the process, it will raise a UsageError with the
	error message.

	:param suppliers: A list of dictionaries containing information about the suppliers.
	:type suppliers: Sequence[SupplierDict]
	:param engine_db: The database engine object.
	:type engine_db: sqlalchemy.engine.Engine
	:raises UsageError: If any error occur during the process of adding suppliers.
	"""
	logger.info("Adding suppliers to database ...")
	with Session(engine_db) as session:
		added = set()
		with Progress(
			TextColumn("[progress.description]{task.description}"),
			BarColumn(),
			TaskProgressColumn(),
			TimeElapsedColumn(),
			transient=True,
		) as progress:
			adder_supp = progress.add_task(description="Adding suppliers to database ...")
			for supplier in suppliers:
				# pprint(f"country_name={supplier['country_name']}")
				if supplier["name"] in added:
					continue
				session.add(
					Supplier(
						name=supplier["name"],
						verification_mode=supplier["verified_type"],
						sopi_level=supplier["sopi_level"],
						country_name=supplier["country_name"],
						years_as_gold_supplier=supplier["gold_supplier_year"],
						supplier_service_score=supplier["supplier_service_score"],
					)
				)
				added.add(supplier["name"])
				try:
					session.commit()
					progress.update(adder_supp, advance=100 / len(suppliers))
				except OperationalError as e:
					if "no such table" in str(e):
						raise UsageError(
							"Your database must be initialized before if you want to update it. instead run :  `aba-run db-init sqlite sqlite-file <your_sqlite_db_file_name>`"
						) from e
					raise UsageError(
						f"Something went wrong an unexpected error has occured:{e}"
					) from e
				except IntegrityError as e:
					raise UsageError(
						"Seems like you are trying to update a database which has already been updated ! Instead initialize a new one and update it with: `aba-run db-update sqlite --sqlite-file <new_sqlite_db_file_name>` "
					) from e


def add_products_to_db(products: Sequence[ProductDict], engine_db: Engine):
	"""
	Add products to the database.

	This function takes two arguments: products and engine_db. products is a list of dictionaries
	containing information about the products. engine_db is the database engine object created by
	create_engine method of SQLAlchemy.

	The function will iterate over the list of products and add each one to the database using the
	Product model. If any error occur during the process, it will raise a UsageError with the
	error message.

	:param products: A list of dictionaries containing information about the products.
	:type products: Sequence[ProductDict]
	:param engine_db: The database engine object.
	:type engine_db: sqlalchemy.engine.Engine
	:raises UsageError: If any error occur during the process of adding products.
	"""
	logger.info("Adding products to database ...")
	with Session(engine_db) as session:
		added = set()
		with Progress(
			SpinnerColumn(),
			TextColumn("[progress.description]{task.description}"),
			BarColumn(),
			TaskProgressColumn(),
			TimeElapsedColumn(),
			transient=True,
		) as progress:
			try:
				add_prods = progress.add_task(description="Adding products to database ...")
				for product in products:
					if product["name"] in added:
						continue
					supplier = _related_supplier(session, supplied_by=product["supplied_by"])
					session.add(
						Product(
							name=product["name"],
							alibaba_guranteed=product["guaranteed_by_alibaba"],
							certifications=product["certifications"],
							minimum_to_order=product["minimum_to_order"],
							ordered_or_sold=product["ordered_or_sold"],
							supplier_id=supplier.id,
							min_price=product["min_price"],
							max_price=product["max_price"],
							trade_product=product["trade_product"],
							review_count=product["review_count"],
							review_score=product["review_score"],
							product_score=product["product_score"],
							shipping_time_score=product["shipping_time_score"],
							is_instant_order=product["instant_order"],
							is_customizable=product["customizable"],
							is_full_promotion=product["is_full_promotion"],
						)
					)
					added.add(product["name"])
					session.commit()
					progress.update(add_prods, advance=100 / len(products))
			except OperationalError as e:
				logger.error(f"Errors has occured: {e}")
				if "no such table" in str(e):
					raise UsageError(
						"Your database must be initialized first run : aba db-init --help to know more "
					) from e
				raise UsageError(f"Something went wrong an unexpected error has occured:{e}") from e


def _related_supplier(current_session: Session, supplied_by: str):
	"""
	Retrieve the supplier related to the given product.

	:param current_session: The database session object.
	:type current_session: sqlalchemy.orm.Session
	:param supplied_by: The name of the supplier.
	:type supplied_by: str
	:return: A Supplier object.
	:rtype: Supplier
	"""
	logger.info("Getting related supplier ...")
	query_statement = select(Supplier).where(Supplier.name == supplied_by)
	return current_session.exec(query_statement).all()[0]


if __name__ == "__main__":
	...



import os

from .ascii import get_current_version

from dotenv import load_dotenv

__version__ = get_current_version()

load_dotenv(override=True)
BRIGHT_DATA_API_KEY: str = os.environ.get("BRIGHT_DATA_API_KEY", "")
SYPHOON_API_KEY: str = os.environ.get("SYPHOON_API_KEY", "")
LOGURU_LEVEL: str | None = os.environ.get("LOGURU_LEVEL")



import asyncio
from asyncio import TaskGroup
from typing import Optional

from loguru import logger
from playwright.async_api import BrowserContext
from playwright.async_api import Page as AsyncPage

from rich.progress import Progress, TaskID


HTML_PAGE_RESULT = []


async def goto_task(
	url: str,
	context_browser: BrowserContext,
	task: TaskID,
	progress: Progress,
	semaphore: asyncio.Semaphore,
	tg_instance: TaskGroup,
	page: AsyncPage,
	page_results: int,
) -> Optional[str]:
	"""Get HTML content from a URL.

	This function will load a page from a given URL,
	get its HTML content and save it to a global list.
	It will also update a progress bar with the task ID.

	Parameters:
		url (str): The URL of the page to be loaded.
		context_browser (BrowserContext): The browser context to be used.
		task (TaskID): The task ID of the current task.
		progress (Progress): The progress bar to be updated.
		semaphore (asyncio.Semaphore): The semaphore to be used to limit the number of concurrent tasks.
		tg_instance (TaskGroup): The task group instance.
		page (AsyncPage): The page object to be used.
		page_results (int): The number of pages to be scrapped.

	Returns:
		Optional[str]: The HTML content of the page if successful, otherwise None.
	"""
	try:
		async with page:
			logger.info(f"Loading page {url.split('page=')[1]} ... ")
			await page.goto(url, wait_until="domcontentloaded", timeout=0)
			logger.info(f"get response text from web page {url.split('page=')[1]} ... ")
			locator = page.locator("css=div.container")
			html_body = await locator.inner_html(timeout=0)
			progress.start_task(task)
			progress.update(task, advance=100 / page_results)
			global HTML_PAGE_RESULT
			HTML_PAGE_RESULT.append(html_body)
			logger.info(f"Closing page {url.split('page=')[1]} ... ")
	except Exception as e:
		logger.error(f"Error processing page {url.split('page=')[1]}: {e}")
		return None


def urls_pusher(words: str, stop_at: int):
	for i in range(1, stop_at + 1):
		yield f"https://www.alibaba.com/trade/search?spm=a2700.galleryofferlist.0.0.64271d1dZRDxt1&fsb=y&IndexArea=product_en&keywords={words}&tab=all&&page={i}"
